//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
import React, { useEffect } from "react";
import { isNil } from "lodash";
import { libTrackIsOn, isNonEmptyString } from "primeobjects-helper-util";
import { useRef, useState } from "react";
import { useServerSentEvents } from "primeobjects-ui-web-basic/build/cjs/hooks/use-sse";
import TextareaAutosize from "react-autosize-textarea/lib/index";
import { getCurrentPoolUser } from "../context/auth-cognito";
import { useSolution } from "primeobjects-helper-ui/build/cjs/hooks/solution";
import { callApi } from "../context/auth-call-api";
export var AIStream = function (props) {
    var _a, _b, _c;
    var style = props.style, placeholder = props.placeholder, disabled = props.disabled, rows = props.rows, maxRows = props.maxRows, request = props.request;
    var className = (_a = props.className) !== null && _a !== void 0 ? _a : "";
    var textRef = useRef("");
    var _d = useState(undefined), streaming = _d[0], setStreaming = _d[1];
    var _e = useState(""), value = _e[0], setValue = _e[1];
    var start = (_b = props.start) !== null && _b !== void 0 ? _b : request === null || request === void 0 ? void 0 : request.start;
    var apiType = (_c = request === null || request === void 0 ? void 0 : request.apiType) !== null && _c !== void 0 ? _c : "prompt";
    var solution = useSolution();
    var appendValue = function (v) {
        onChange(textRef.current.trimStart() + v);
    };
    var resetValue = function () {
        onChange("");
    };
    var onError = function (error) {
        console.error(error);
        setStreaming(undefined);
        props.onError && props.onError(error);
    };
    var openStream = useServerSentEvents({
        baseUrl: "/api/ai-stream",
        config: {
            withCredentials: false
        },
        onData: function (data) {
            var _a;
            if (libTrackIsOn())
                console.log({ onData: data });
            try {
                var text = (_a = JSON.parse(data).choices[0].text) !== null && _a !== void 0 ? _a : JSON.parse(data).choices[0].delta.content;
                if (isNonEmptyString(text) &&
                    ((textRef.current.length == 0 && text != "\n") ||
                        textRef.current.length > 0)) {
                    appendValue(text);
                    props.onData && props.onData(text);
                }
            }
            catch (err) {
                console.log("Failed to parse data: ".concat(data));
            }
        },
        onOpen: function () {
            if (libTrackIsOn())
                console.log("onOpen");
            props.onStart && props.onStart();
            if (textRef.current) {
                textRef.current = "";
            }
        },
        onClose: function () {
            if (libTrackIsOn())
                console.log("onClose");
            onFinish();
        },
        onError: function (event) {
            onError(event);
        }
    }).openStream;
    var onChange = function (value) {
        textRef.current = value;
        setValue(value);
        props.onChange && props.onChange(value);
    };
    var runStream = function (auth) {
        //because the stream request will fail if there is large content in the assistant
        //we create an S3 file first by calling submit-assistant-stream API, it contains assistant and OpenAI data
        //the stream will get detail from s3 file by calling start-assistant-stream
        callApi(solution, "".concat(solution.apis[apiType], "submit-stream"), 
        //"http://localhost:3000/prod/submit-stream",
        {
            request: request
        }, "POST")
            .then(function (result) {
            openStream({
                query: {
                    auth: auth,
                    transactionId: result.transactionId,
                    apiType: apiType
                }
            });
        })
            .catch(function (error) {
            onError(error);
        });
    };
    var onFinish = function () {
        var _a, _b, _c;
        setStreaming(undefined);
        var result = "".concat(textRef.current);
        props.onFinish && props.onFinish(result);
        var maxTokens = (_a = request === null || request === void 0 ? void 0 : request.maxTokens) !== null && _a !== void 0 ? _a : 0;
        var commandTokens = ("".concat((_b = request === null || request === void 0 ? void 0 : request.template) !== null && _b !== void 0 ? _b : "").length +
            JSON.stringify((_c = request === null || request === void 0 ? void 0 : request.parameters) !== null && _c !== void 0 ? _c : {}).length) /
            4;
        var estimatedTokens = commandTokens + maxTokens;
        var realTokens = commandTokens + result.length / 4;
        console.log({
            estimatedTokens: estimatedTokens,
            request: request,
            commandTokens: commandTokens,
            r: result.length
        });
        callApi(solution, "".concat(solution.apis[apiType], "finish-stream"), 
        //"http://localhost:3000/prod/finish-stream",
        {
            model: request === null || request === void 0 ? void 0 : request.model,
            transactionId: request === null || request === void 0 ? void 0 : request.transactionId,
            tokens: realTokens - estimatedTokens
        }, "POST");
    };
    useEffect(function () {
        if (!isNil(start) && start != streaming) {
            resetValue();
            setStreaming(start);
            getCurrentPoolUser(props.solution ? props.solution : solution).then(function (cognito) {
                if (cognito) {
                    var authorization = cognito.signInUserSession.idToken.jwtToken;
                    var accessToken = cognito.signInUserSession.accessToken.jwtToken;
                    var auth = { authorization: authorization, accessToken: accessToken };
                    runStream(JSON.stringify(auth));
                }
            });
        }
    }, [start]);
    return (React.createElement(TextareaAutosize, { value: value, style: style, rows: rows, maxRows: maxRows, disabled: disabled, className: className, placeholder: placeholder, onChange: function (e) {
            if (textRef.current != e.target.value) {
                onChange(e.target.value);
            }
        } }));
};
export default AIStream;
//# sourceMappingURL=ai-stream.js.map